@@ -3,7 +3,7 @@
 The API and results of this estimator might change without any deprecation
 cycle.
 
-Importing this file dynamically sets :class:`sklearn.impute.IterativeImputer`
+Importing this file dynamically sets :class:`~sklearn.impute.IterativeImputer`
 as an attribute of the impute module::
 
     >>> # explicitly require this experimental feature
