@@ -1,5 +1,6 @@
 
 from functools import partial
+from inspect import signature
 from itertools import product
 from itertools import chain
 from itertools import permutations
@@ -16,14 +17,11 @@ from sklearn.utils.validation import _num_samples
 from sklearn.utils.validation import check_random_state
 from sklearn.utils import shuffle
 
-from sklearn.utils.testing import assert_allclose
-from sklearn.utils.testing import assert_almost_equal
-from sklearn.utils.testing import assert_array_equal
-from sklearn.utils.testing import assert_array_less
-from sklearn.utils.testing import assert_equal
-from sklearn.utils.testing import assert_raise_message
-from sklearn.utils.testing import assert_raises
-from sklearn.utils.testing import ignore_warnings
+from sklearn.utils._testing import assert_allclose
+from sklearn.utils._testing import assert_almost_equal
+from sklearn.utils._testing import assert_array_equal
+from sklearn.utils._testing import assert_array_less
+from sklearn.utils._testing import ignore_warnings
 
 from sklearn.metrics import accuracy_score
 from sklearn.metrics import average_precision_score
@@ -32,6 +30,7 @@ from sklearn.metrics import brier_score_loss
 from sklearn.metrics import cohen_kappa_score
 from sklearn.metrics import confusion_matrix
 from sklearn.metrics import coverage_error
+from sklearn.metrics import det_curve
 from sklearn.metrics import explained_variance_score
 from sklearn.metrics import f1_score
 from sklearn.metrics import fbeta_score
@@ -44,7 +43,11 @@ from sklearn.metrics import log_loss
 from sklearn.metrics import max_error
 from sklearn.metrics import matthews_corrcoef
 from sklearn.metrics import mean_absolute_error
+from sklearn.metrics import mean_absolute_percentage_error
 from sklearn.metrics import mean_squared_error
+from sklearn.metrics import mean_tweedie_deviance
+from sklearn.metrics import mean_poisson_deviance
+from sklearn.metrics import mean_gamma_deviance
 from sklearn.metrics import median_absolute_error
 from sklearn.metrics import multilabel_confusion_matrix
 from sklearn.metrics import precision_recall_curve
@@ -54,8 +57,11 @@ from sklearn.metrics import recall_score
 from sklearn.metrics import roc_auc_score
 from sklearn.metrics import roc_curve
 from sklearn.metrics import zero_one_loss
+from sklearn.metrics import ndcg_score
+from sklearn.metrics import dcg_score
+from sklearn.metrics import top_k_accuracy_score
 
-from sklearn.metrics.base import _average_binary_score
+from sklearn.metrics._base import _average_binary_score
 
 
 # Note toward developers about metric testing
@@ -96,8 +102,14 @@ REGRESSION_METRICS = {
     "mean_absolute_error": mean_absolute_error,
     "mean_squared_error": mean_squared_error,
     "median_absolute_error": median_absolute_error,
+    "mean_absolute_percentage_error": mean_absolute_percentage_error,
     "explained_variance_score": explained_variance_score,
     "r2_score": partial(r2_score, multioutput='variance_weighted'),
+    "mean_normal_deviance": partial(mean_tweedie_deviance, power=0),
+    "mean_poisson_deviance": mean_poisson_deviance,
+    "mean_gamma_deviance": mean_gamma_deviance,
+    "mean_compound_poisson_deviance":
+    partial(mean_tweedie_deviance, power=1.4),
 }
 
 CLASSIFICATION_METRICS = {
@@ -108,7 +120,7 @@ CLASSIFICATION_METRICS = {
     "unnormalized_accuracy_score": partial(accuracy_score, normalize=False),
 
     # `confusion_matrix` returns absolute values and hence behaves unnormalized
-    # . Naming it with an unnormalized_ prefix is neccessary for this module to
+    # . Naming it with an unnormalized_ prefix is necessary for this module to
     # skip sample_weight scaling checks which will fail for unnormalized
     # metrics.
     "unnormalized_confusion_matrix": confusion_matrix,
@@ -186,7 +198,7 @@ def precision_recall_curve_padded_thresholds(*args, **kwargs):
     return np.array([
         precision,
         recall,
-        np.pad(thresholds,
+        np.pad(thresholds.astype(np.float64),
                pad_width=(0, pad_threshholds),
                mode='constant',
                constant_values=[np.nan])
@@ -196,6 +208,7 @@ def precision_recall_curve_padded_thresholds(*args, **kwargs):
 CURVE_METRICS = {
     "roc_curve": roc_curve,
     "precision_recall_curve": precision_recall_curve_padded_thresholds,
+    "det_curve": det_curve,
 }
 
 THRESHOLDED_METRICS = {
@@ -212,6 +225,12 @@ THRESHOLDED_METRICS = {
     "weighted_roc_auc": partial(roc_auc_score, average="weighted"),
     "samples_roc_auc": partial(roc_auc_score, average="samples"),
     "micro_roc_auc": partial(roc_auc_score, average="micro"),
+    "ovr_roc_auc": partial(roc_auc_score, average="macro", multi_class='ovr'),
+    "weighted_ovr_roc_auc": partial(roc_auc_score, average="weighted",
+                                    multi_class='ovr'),
+    "ovo_roc_auc": partial(roc_auc_score, average="macro", multi_class='ovo'),
+    "weighted_ovo_roc_auc": partial(roc_auc_score, average="weighted",
+                                    multi_class='ovo'),
     "partial_roc_auc": partial(roc_auc_score, max_fpr=0.5),
 
     "average_precision_score":
@@ -224,6 +243,10 @@ THRESHOLDED_METRICS = {
     partial(average_precision_score, average="micro"),
     "label_ranking_average_precision_score":
     label_ranking_average_precision_score,
+    "ndcg_score": ndcg_score,
+    "dcg_score": dcg_score,
+
+    "top_k_accuracy_score": top_k_accuracy_score
 }
 
 ALL_METRICS = dict()
@@ -253,17 +276,19 @@ METRIC_UNDEFINED_BINARY = {
     "unnormalized_multilabel_confusion_matrix_sample",
     "label_ranking_loss",
     "label_ranking_average_precision_score",
+    "dcg_score",
+    "ndcg_score"
 }
 
 # Those metrics don't support multiclass inputs
 METRIC_UNDEFINED_MULTICLASS = {
     "brier_score_loss",
 
-    "roc_auc_score",
     "micro_roc_auc",
-    "weighted_roc_auc",
     "samples_roc_auc",
     "partial_roc_auc",
+    "roc_auc_score",
+    "weighted_roc_auc",
 
     "average_precision_score",
     "weighted_average_precision_score",
@@ -282,6 +307,7 @@ METRIC_UNDEFINED_MULTICLASS = {
     # curves
     "roc_curve",
     "precision_recall_curve",
+    "det_curve",
 }
 
 # Metric undefined with "binary" or "multiclass" input
@@ -303,6 +329,7 @@ THRESHOLDED_METRICS_WITH_AVERAGING = {
 METRICS_WITH_POS_LABEL = {
     "roc_curve",
     "precision_recall_curve",
+    "det_curve",
 
     "brier_score_loss",
 
@@ -333,8 +360,7 @@ METRICS_WITH_LABELS = {
     "normalized_confusion_matrix",
     "roc_curve",
     "precision_recall_curve",
-
-    "hamming_loss",
+    "det_curve",
 
     "precision_score", "recall_score", "f1_score", "f2_score", "f0.5_score",
     "jaccard_score",
@@ -360,6 +386,7 @@ METRICS_WITH_LABELS = {
 # Metrics with a "normalize" option
 METRICS_WITH_NORMALIZE_OPTION = {
     "accuracy_score",
+    "top_k_accuracy_score",
     "zero_one_loss",
 }
 
@@ -375,6 +402,10 @@ THRESHOLDED_MULTILABEL_METRICS = {
     "samples_average_precision_score", "micro_average_precision_score",
 
     "coverage_error", "label_ranking_loss",
+
+    "ndcg_score",
+    "dcg_score",
+
     "label_ranking_average_precision_score",
 }
 
@@ -405,8 +436,8 @@ MULTILABELS_METRICS = {
 
 # Regression metrics with "multioutput-continuous" format support
 MULTIOUTPUT_METRICS = {
-    "mean_absolute_error", "mean_squared_error", "r2_score",
-    "explained_variance_score"
+    "mean_absolute_error", "median_absolute_error", "mean_squared_error",
+    "r2_score", "explained_variance_score", "mean_absolute_percentage_error"
 }
 
 # Symmetric with respect to their input arguments y_true and y_pred
@@ -429,7 +460,7 @@ SYMMETRIC_METRICS = {
     "matthews_corrcoef_score", "mean_absolute_error", "mean_squared_error",
     "median_absolute_error", "max_error",
 
-    "cohen_kappa_score",
+    "cohen_kappa_score", "mean_normal_deviance"
 }
 
 # Asymmetric with respect to their input arguments y_true and y_pred
@@ -443,6 +474,7 @@ NOT_SYMMETRIC_METRICS = {
     "normalized_confusion_matrix",
     "roc_curve",
     "precision_recall_curve",
+    "det_curve",
 
     "precision_score", "recall_score", "f2_score", "f0.5_score",
 
@@ -451,69 +483,104 @@ NOT_SYMMETRIC_METRICS = {
     "unnormalized_multilabel_confusion_matrix",
 
     "macro_f0.5_score", "macro_f2_score", "macro_precision_score",
-    "macro_recall_score", "log_loss", "hinge_loss"
+    "macro_recall_score", "log_loss", "hinge_loss",
+    "mean_gamma_deviance", "mean_poisson_deviance",
+    "mean_compound_poisson_deviance", "mean_absolute_percentage_error"
 }
 
 
 # No Sample weight support
 METRICS_WITHOUT_SAMPLE_WEIGHT = {
     "median_absolute_error",
-    "max_error"
+    "max_error",
+    "ovo_roc_auc",
+    "weighted_ovo_roc_auc"
+}
+
+METRICS_REQUIRE_POSITIVE_Y = {
+    "mean_poisson_deviance",
+    "mean_gamma_deviance",
+    "mean_compound_poisson_deviance",
 }
 
 
-@ignore_warnings
-def test_symmetry():
+def _require_positive_targets(y1, y2):
+    """Make targets strictly positive"""
+    offset = abs(min(y1.min(), y2.min())) + 1
+    y1 += offset
+    y2 += offset
+    return y1, y2
+
+
+def test_symmetry_consistency():
+
+    # We shouldn't forget any metrics
+    assert (SYMMETRIC_METRICS.union(
+        NOT_SYMMETRIC_METRICS, set(THRESHOLDED_METRICS),
+        METRIC_UNDEFINED_BINARY_MULTICLASS) ==
+        set(ALL_METRICS))
+
+    assert (
+        SYMMETRIC_METRICS.intersection(NOT_SYMMETRIC_METRICS) ==
+        set())
+
+
+@pytest.mark.parametrize("name", sorted(SYMMETRIC_METRICS))
+def test_symmetric_metric(name):
     # Test the symmetry of score and loss functions
     random_state = check_random_state(0)
     y_true = random_state.randint(0, 2, size=(20, ))
     y_pred = random_state.randint(0, 2, size=(20, ))
 
+    if name in METRICS_REQUIRE_POSITIVE_Y:
+        y_true, y_pred = _require_positive_targets(y_true, y_pred)
+
     y_true_bin = random_state.randint(0, 2, size=(20, 25))
     y_pred_bin = random_state.randint(0, 2, size=(20, 25))
 
-    # We shouldn't forget any metrics
-    assert_equal(SYMMETRIC_METRICS.union(
-        NOT_SYMMETRIC_METRICS, set(THRESHOLDED_METRICS),
-        METRIC_UNDEFINED_BINARY_MULTICLASS),
-        set(ALL_METRICS))
-
-    assert_equal(
-        SYMMETRIC_METRICS.intersection(NOT_SYMMETRIC_METRICS),
-        set())
-
-    # Symmetric metric
-    for name in SYMMETRIC_METRICS:
-        metric = ALL_METRICS[name]
-        if name in METRIC_UNDEFINED_BINARY:
-            if name in MULTILABELS_METRICS:
-                assert_allclose(metric(y_true_bin, y_pred_bin),
-                                metric(y_pred_bin, y_true_bin),
-                                err_msg="%s is not symmetric" % name)
-            else:
-                assert False, "This case is currently unhandled"
-        else:
-            assert_allclose(metric(y_true, y_pred),
-                            metric(y_pred, y_true),
+    metric = ALL_METRICS[name]
+    if name in METRIC_UNDEFINED_BINARY:
+        if name in MULTILABELS_METRICS:
+            assert_allclose(metric(y_true_bin, y_pred_bin),
+                            metric(y_pred_bin, y_true_bin),
                             err_msg="%s is not symmetric" % name)
+        else:
+            assert False, "This case is currently unhandled"
+    else:
+        assert_allclose(metric(y_true, y_pred),
+                        metric(y_pred, y_true),
+                        err_msg="%s is not symmetric" % name)
 
-    # Not symmetric metrics
-    for name in NOT_SYMMETRIC_METRICS:
-        metric = ALL_METRICS[name]
 
-        # use context manager to supply custom error message
-        with assert_raises(AssertionError) as cm:
-            assert_array_equal(metric(y_true, y_pred), metric(y_pred, y_true))
-            cm.msg = ("%s seems to be symmetric" % name)
+@pytest.mark.parametrize("name", sorted(NOT_SYMMETRIC_METRICS))
+def test_not_symmetric_metric(name):
+    # Test the symmetry of score and loss functions
+    random_state = check_random_state(0)
+    y_true = random_state.randint(0, 2, size=(20, ))
+    y_pred = random_state.randint(0, 2, size=(20, ))
+
+    if name in METRICS_REQUIRE_POSITIVE_Y:
+        y_true, y_pred = _require_positive_targets(y_true, y_pred)
+
+    metric = ALL_METRICS[name]
+
+    # use context manager to supply custom error message
+    with pytest.raises(AssertionError):
+        assert_array_equal(metric(y_true, y_pred), metric(y_pred, y_true))
+        raise ValueError("%s seems to be symmetric" % name)
 
 
 @pytest.mark.parametrize(
         'name',
-        set(ALL_METRICS) - METRIC_UNDEFINED_BINARY_MULTICLASS)
+        sorted(set(ALL_METRICS) - METRIC_UNDEFINED_BINARY_MULTICLASS))
 def test_sample_order_invariance(name):
     random_state = check_random_state(0)
     y_true = random_state.randint(0, 2, size=(20, ))
     y_pred = random_state.randint(0, 2, size=(20, ))
+
+    if name in METRICS_REQUIRE_POSITIVE_Y:
+        y_true, y_pred = _require_positive_targets(y_true, y_pred)
+
     y_true_shuffle, y_pred_shuffle = shuffle(y_true, y_pred, random_state=0)
 
     with ignore_warnings():
@@ -561,12 +628,15 @@ def test_sample_order_invariance_multilabel_and_multioutput():
 
 @pytest.mark.parametrize(
         'name',
-        set(ALL_METRICS) - METRIC_UNDEFINED_BINARY_MULTICLASS)
+        sorted(set(ALL_METRICS) - METRIC_UNDEFINED_BINARY_MULTICLASS))
 def test_format_invariance_with_1d_vectors(name):
     random_state = check_random_state(0)
     y1 = random_state.randint(0, 2, size=(20, ))
     y2 = random_state.randint(0, 2, size=(20, ))
 
+    if name in METRICS_REQUIRE_POSITIVE_Y:
+        y1, y2 = _require_positive_targets(y1, y2)
+
     y1_list = list(y1)
     y2_list = list(y2)
 
@@ -621,23 +691,30 @@ def test_format_invariance_with_1d_vectors(name):
                                 "list and np-array-column" % name)
 
         # These mix representations aren't allowed
-        assert_raises(ValueError, metric, y1_1d, y2_row)
-        assert_raises(ValueError, metric, y1_row, y2_1d)
-        assert_raises(ValueError, metric, y1_list, y2_row)
-        assert_raises(ValueError, metric, y1_row, y2_list)
-        assert_raises(ValueError, metric, y1_column, y2_row)
-        assert_raises(ValueError, metric, y1_row, y2_column)
+        with pytest.raises(ValueError):
+            metric(y1_1d, y2_row)
+        with pytest.raises(ValueError):
+            metric(y1_row, y2_1d)
+        with pytest.raises(ValueError):
+            metric(y1_list, y2_row)
+        with pytest.raises(ValueError):
+            metric(y1_row, y2_list)
+        with pytest.raises(ValueError):
+            metric(y1_column, y2_row)
+        with pytest.raises(ValueError):
+            metric(y1_row, y2_column)
 
         # NB: We do not test for y1_row, y2_row as these may be
         # interpreted as multilabel or multioutput data.
         if (name not in (MULTIOUTPUT_METRICS | THRESHOLDED_MULTILABEL_METRICS |
                          MULTILABELS_METRICS)):
-            assert_raises(ValueError, metric, y1_row, y2_row)
+            with pytest.raises(ValueError):
+                metric(y1_row, y2_row)
 
 
 @pytest.mark.parametrize(
-       'name',
-       set(CLASSIFICATION_METRICS) - METRIC_UNDEFINED_BINARY_MULTICLASS)
+    'name',
+    sorted(set(CLASSIFICATION_METRICS) - METRIC_UNDEFINED_BINARY_MULTICLASS))
 def test_classification_invariance_string_vs_numbers_labels(name):
     # Ensure that classification metrics with string labels are invariant
     random_state = check_random_state(0)
@@ -716,34 +793,59 @@ def test_thresholded_invariance_string_vs_numbers_labels(name):
                                        "invariance test".format(name))
         else:
             # TODO those metrics doesn't support string label yet
-            assert_raises(ValueError, metric, y1_str, y2)
-            assert_raises(ValueError, metric, y1_str.astype('O'), y2)
+            with pytest.raises(ValueError):
+                metric(y1_str, y2)
+            with pytest.raises(ValueError):
+                metric(y1_str.astype('O'), y2)
 
 
-invalids = [([0, 1], [np.inf, np.inf]),
-            ([0, 1], [np.nan, np.nan]),
-            ([0, 1], [np.nan, np.inf])]
+invalids_nan_inf = [
+    ([0, 1], [np.inf, np.inf]),
+    ([0, 1], [np.nan, np.nan]),
+    ([0, 1], [np.nan, np.inf]),
+    ([0, 1], [np.inf, 1]),
+    ([0, 1], [np.nan, 1]),
+]
 
 
 @pytest.mark.parametrize(
-        'metric',
-        chain(THRESHOLDED_METRICS.values(), REGRESSION_METRICS.values()))
-def test_regression_thresholded_inf_nan_input(metric):
-
-    for y_true, y_score in invalids:
-        assert_raise_message(ValueError,
-                             "contains NaN, infinity",
-                             metric, y_true, y_score)
+    'metric',
+    chain(THRESHOLDED_METRICS.values(), REGRESSION_METRICS.values())
+)
+@pytest.mark.parametrize("y_true, y_score", invalids_nan_inf)
+def test_regression_thresholded_inf_nan_input(metric, y_true, y_score):
+    with pytest.raises(ValueError, match="contains NaN, infinity"):
+        metric(y_true, y_score)
 
 
 @pytest.mark.parametrize('metric', CLASSIFICATION_METRICS.values())
-def test_classification_inf_nan_input(metric):
-    # Classification metrics all raise a mixed input exception
-    for y_true, y_score in invalids:
-        assert_raise_message(ValueError,
-                             "Input contains NaN, infinity or a "
-                             "value too large",
-                             metric, y_true, y_score)
+@pytest.mark.parametrize(
+    "y_true, y_score",
+    invalids_nan_inf +
+    # Add an additional case for classification only
+    # non-regression test for:
+    # https://github.com/scikit-learn/scikit-learn/issues/6809
+    [([np.nan, 1, 2], [1, 2, 3])]
+)
+def test_classification_inf_nan_input(metric, y_true, y_score):
+    """check that classification metrics raise a message mentioning the
+    occurrence of non-finite values in the target vectors."""
+    err_msg = "Input contains NaN, infinity or a value too large"
+    with pytest.raises(ValueError, match=err_msg):
+        metric(y_true, y_score)
+
+
+@pytest.mark.parametrize('metric', CLASSIFICATION_METRICS.values())
+def test_classification_binary_continuous_input(metric):
+    """check that classification metrics raise a message of mixed type data
+    with continuous/binary target vectors."""
+    y_true, y_score = ['a', 'b', 'a'], [0.1, 0.2, 0.3]
+    err_msg = (
+        "Classification metrics can't handle a mix of binary and continuous "
+        "targets"
+    )
+    with pytest.raises(ValueError, match=err_msg):
+        metric(y_true, y_score)
 
 
 @ignore_warnings
@@ -755,7 +857,11 @@ def check_single_sample(name):
     metric = ALL_METRICS[name]
 
     # assert that no exception is thrown
-    for i, j in product([0, 1], repeat=2):
+    if name in METRICS_REQUIRE_POSITIVE_Y:
+        values = [1, 2]
+    else:
+        values = [0, 1]
+    for i, j in product(values, repeat=2):
         metric([i], [j])
 
 
@@ -767,31 +873,33 @@ def check_single_sample_multioutput(name):
 
 
 @pytest.mark.parametrize(
-        'name',
-        (set(ALL_METRICS)
-         # Those metrics are not always defined with one sample
-         # or in multiclass classification
-         - METRIC_UNDEFINED_BINARY_MULTICLASS
-         - set(THRESHOLDED_METRICS)))
+    'name',
+    sorted(
+        set(ALL_METRICS)
+        # Those metrics are not always defined with one sample
+        # or in multiclass classification
+        - METRIC_UNDEFINED_BINARY_MULTICLASS - set(THRESHOLDED_METRICS)))
 def test_single_sample(name):
     check_single_sample(name)
 
 
-@pytest.mark.parametrize('name', MULTIOUTPUT_METRICS | MULTILABELS_METRICS)
+@pytest.mark.parametrize('name',
+                         sorted(MULTIOUTPUT_METRICS | MULTILABELS_METRICS))
 def test_single_sample_multioutput(name):
     check_single_sample_multioutput(name)
 
 
-@pytest.mark.parametrize('name', MULTIOUTPUT_METRICS)
+@pytest.mark.parametrize('name', sorted(MULTIOUTPUT_METRICS))
 def test_multioutput_number_of_output_differ(name):
     y_true = np.array([[1, 0, 0, 1], [0, 1, 1, 1], [1, 1, 0, 1]])
     y_pred = np.array([[0, 0], [1, 0], [0, 0]])
 
     metric = ALL_METRICS[name]
-    assert_raises(ValueError, metric, y_true, y_pred)
+    with pytest.raises(ValueError):
+        metric(y_true, y_pred)
 
 
-@pytest.mark.parametrize('name', MULTIOUTPUT_METRICS)
+@pytest.mark.parametrize('name', sorted(MULTIOUTPUT_METRICS))
 def test_multioutput_regression_invariance_to_dimension_shuffling(name):
     # test invariance to dimension shuffling
     random_state = check_random_state(0)
@@ -829,6 +937,12 @@ def test_multilabel_representation_invariance():
     y1_sparse_indicator = sp.coo_matrix(y1)
     y2_sparse_indicator = sp.coo_matrix(y2)
 
+    y1_list_array_indicator = list(y1)
+    y2_list_array_indicator = list(y2)
+
+    y1_list_list_indicator = [list(a) for a in y1_list_array_indicator]
+    y2_list_list_indicator = [list(a) for a in y2_list_array_indicator]
+
     for name in MULTILABELS_METRICS:
         metric = ALL_METRICS[name]
 
@@ -844,13 +958,24 @@ def test_multilabel_representation_invariance():
                         measure,
                         err_msg="%s failed representation invariance between "
                                 "dense and sparse indicator formats." % name)
+        assert_almost_equal(metric(y1_list_list_indicator,
+                                   y2_list_list_indicator),
+                            measure,
+                            err_msg="%s failed representation invariance  "
+                                    "between dense array and list of list "
+                                    "indicator formats." % name)
+        assert_almost_equal(metric(y1_list_array_indicator,
+                                   y2_list_array_indicator),
+                            measure,
+                            err_msg="%s failed representation invariance  "
+                                    "between dense and list of array "
+                                    "indicator formats." % name)
 
 
-@pytest.mark.parametrize('name', MULTILABELS_METRICS)
+@pytest.mark.parametrize('name', sorted(MULTILABELS_METRICS))
 def test_raise_value_error_multilabel_sequences(name):
     # make sure the multilabel-sequence format raises ValueError
     multilabel_sequences = [
-        [[0, 1]],
         [[1], [2], [0, 1]],
         [(), (2), (0, 1)],
         [[]],
@@ -859,47 +984,66 @@ def test_raise_value_error_multilabel_sequences(name):
 
     metric = ALL_METRICS[name]
     for seq in multilabel_sequences:
-        assert_raises(ValueError, metric, seq, seq)
+        with pytest.raises(ValueError):
+            metric(seq, seq)
 
 
-@pytest.mark.parametrize('name', METRICS_WITH_NORMALIZE_OPTION)
+@pytest.mark.parametrize('name', sorted(METRICS_WITH_NORMALIZE_OPTION))
 def test_normalize_option_binary_classification(name):
     # Test in the binary case
+    n_classes = 2
     n_samples = 20
     random_state = check_random_state(0)
-    y_true = random_state.randint(0, 2, size=(n_samples, ))
-    y_pred = random_state.randint(0, 2, size=(n_samples, ))
+
+    y_true = random_state.randint(0, n_classes, size=(n_samples, ))
+    y_pred = random_state.randint(0, n_classes, size=(n_samples, ))
+    y_score = random_state.normal(size=y_true.shape)
 
     metrics = ALL_METRICS[name]
-    measure = metrics(y_true, y_pred, normalize=True)
-    assert_array_less(-1.0 * measure, 0,
+    pred = y_score if name in THRESHOLDED_METRICS else y_pred
+    measure_normalized = metrics(y_true, pred, normalize=True)
+    measure_not_normalized = metrics(y_true, pred, normalize=False)
+
+    assert_array_less(-1.0 * measure_normalized, 0,
                       err_msg="We failed to test correctly the normalize "
                               "option")
-    assert_allclose(metrics(y_true, y_pred, normalize=False) / n_samples,
-                    measure)
+
+    assert_allclose(measure_normalized, measure_not_normalized / n_samples,
+                    err_msg=f"Failed with {name}")
 
 
-@pytest.mark.parametrize('name', METRICS_WITH_NORMALIZE_OPTION)
+@pytest.mark.parametrize('name', sorted(METRICS_WITH_NORMALIZE_OPTION))
 def test_normalize_option_multiclass_classification(name):
     # Test in the multiclass case
+    n_classes = 4
+    n_samples = 20
     random_state = check_random_state(0)
-    y_true = random_state.randint(0, 4, size=(20, ))
-    y_pred = random_state.randint(0, 4, size=(20, ))
-    n_samples = y_true.shape[0]
+
+    y_true = random_state.randint(0, n_classes, size=(n_samples, ))
+    y_pred = random_state.randint(0, n_classes, size=(n_samples, ))
+    y_score = random_state.uniform(size=(n_samples, n_classes))
 
     metrics = ALL_METRICS[name]
-    measure = metrics(y_true, y_pred, normalize=True)
-    assert_array_less(-1.0 * measure, 0,
+    pred = y_score if name in THRESHOLDED_METRICS else y_pred
+    measure_normalized = metrics(y_true, pred, normalize=True)
+    measure_not_normalized = metrics(y_true, pred, normalize=False)
+
+    assert_array_less(-1.0 * measure_normalized, 0,
                       err_msg="We failed to test correctly the normalize "
                               "option")
-    assert_allclose(metrics(y_true, y_pred, normalize=False) / n_samples,
-                    measure)
+
+    assert_allclose(measure_normalized, measure_not_normalized / n_samples,
+                    err_msg=f"Failed with {name}")
 
 
-def test_normalize_option_multilabel_classification():
+@pytest.mark.parametrize('name', sorted(
+    METRICS_WITH_NORMALIZE_OPTION.intersection(MULTILABELS_METRICS)
+))
+def test_normalize_option_multilabel_classification(name):
     # Test in the multilabel case
     n_classes = 4
     n_samples = 100
+    random_state = check_random_state(0)
 
     # for both random_state 0 and 1, y_true and y_pred has at least one
     # unlabelled entry
@@ -914,18 +1058,23 @@ def test_normalize_option_multilabel_classification():
                                                allow_unlabeled=True,
                                                n_samples=n_samples)
 
+    y_score = random_state.uniform(size=y_true.shape)
+
     # To make sure at least one empty label is present
     y_true += [0]*n_classes
     y_pred += [0]*n_classes
 
-    for name in METRICS_WITH_NORMALIZE_OPTION:
-        metrics = ALL_METRICS[name]
-        measure = metrics(y_true, y_pred, normalize=True)
-        assert_array_less(-1.0 * measure, 0,
-                          err_msg="We failed to test correctly the normalize "
-                                  "option")
-        assert_allclose(metrics(y_true, y_pred, normalize=False) / n_samples,
-                        measure, err_msg="Failed with %s" % name)
+    metrics = ALL_METRICS[name]
+    pred = y_score if name in THRESHOLDED_METRICS else y_pred
+    measure_normalized = metrics(y_true, pred, normalize=True)
+    measure_not_normalized = metrics(y_true, pred, normalize=False)
+
+    assert_array_less(-1.0 * measure_normalized, 0,
+                      err_msg="We failed to test correctly the normalize "
+                              "option")
+
+    assert_allclose(measure_normalized, measure_not_normalized / n_samples,
+                    err_msg=f"Failed with {name}")
 
 
 @ignore_warnings
@@ -966,8 +1115,10 @@ def _check_averaging(metric, y_true, y_pred, y_true_binarize, y_pred_binarize,
                         np.mean([metric(y_true_binarize[i], y_pred_binarize[i])
                                  for i in range(n_samples)]))
 
-    assert_raises(ValueError, metric, y_true, y_pred, average="unknown")
-    assert_raises(ValueError, metric, y_true, y_pred, average="garbage")
+    with pytest.raises(ValueError):
+        metric(y_true, y_pred, average="unknown")
+    with pytest.raises(ValueError):
+        metric(y_true, y_pred, average="garbage")
 
 
 def check_averaging(name, y_true, y_true_binarize, y_pred, y_pred_binarize,
@@ -986,7 +1137,7 @@ def check_averaging(name, y_true, y_true_binarize, y_pred, y_pred_binarize,
         raise ValueError("Metric is not recorded as having an average option")
 
 
-@pytest.mark.parametrize('name', METRICS_WITH_AVERAGING)
+@pytest.mark.parametrize('name', sorted(METRICS_WITH_AVERAGING))
 def test_averaging_multiclass(name):
     n_samples, n_classes = 50, 3
     random_state = check_random_state(0)
@@ -1003,7 +1154,8 @@ def test_averaging_multiclass(name):
 
 
 @pytest.mark.parametrize(
-        'name', METRICS_WITH_AVERAGING | THRESHOLDED_METRICS_WITH_AVERAGING)
+    'name',
+    sorted(METRICS_WITH_AVERAGING | THRESHOLDED_METRICS_WITH_AVERAGING))
 def test_averaging_multilabel(name):
     n_samples, n_classes = 40, 5
     _, y = make_multilabel_classification(n_features=1, n_classes=n_classes,
@@ -1019,7 +1171,7 @@ def test_averaging_multilabel(name):
                     y_pred, y_pred_binarize, y_score)
 
 
-@pytest.mark.parametrize('name', METRICS_WITH_AVERAGING)
+@pytest.mark.parametrize('name', sorted(METRICS_WITH_AVERAGING))
 def test_averaging_multilabel_all_zeroes(name):
     y_true = np.zeros((20, 3))
     y_pred = np.zeros((20, 3))
@@ -1044,7 +1196,7 @@ def test_averaging_binary_multilabel_all_zeroes():
                      y_pred_binarize, is_multilabel=True)
 
 
-@pytest.mark.parametrize('name', METRICS_WITH_AVERAGING)
+@pytest.mark.parametrize('name', sorted(METRICS_WITH_AVERAGING))
 def test_averaging_multilabel_all_ones(name):
     y_true = np.ones((20, 3))
     y_pred = np.ones((20, 3))
@@ -1061,6 +1213,10 @@ def check_sample_weight_invariance(name, metric, y1, y2):
     rng = np.random.RandomState(0)
     sample_weight = rng.randint(1, 10, size=len(y1))
 
+    # top_k_accuracy_score always lead to a perfect score for k > 1 in the
+    # binary case
+    metric = partial(metric, k=1) if name == "top_k_accuracy_score" else metric
+
     # check that unit weights gives the same score as no weight
     unweighted_score = metric(y1, y2, sample_weight=None)
 
@@ -1074,11 +1230,11 @@ def check_sample_weight_invariance(name, metric, y1, y2):
     weighted_score = metric(y1, y2, sample_weight=sample_weight)
 
     # use context manager to supply custom error message
-    with assert_raises(AssertionError) as cm:
+    with pytest.raises(AssertionError):
         assert_allclose(unweighted_score, weighted_score)
-        cm.msg = ("Unweighted and weighted scores are unexpectedly almost "
-                  "equal (%s) and (%s) for %s" % (unweighted_score,
-                                                  weighted_score, name))
+        raise ValueError("Unweighted and weighted scores are unexpectedly "
+                         "almost equal (%s) and (%s) "
+                         "for %s" % (unweighted_score, weighted_score, name))
 
     # check that sample_weight can be a list
     weighted_score_list = metric(y1, y2,
@@ -1126,19 +1282,20 @@ def check_sample_weight_invariance(name, metric, y1, y2):
 
     # Check that if number of samples in y_true and sample_weight are not
     # equal, meaningful error is raised.
-    error_message = ("Found input variables with inconsistent numbers of "
-                     "samples: [{}, {}, {}]".format(
+    error_message = (r"Found input variables with inconsistent numbers of "
+                     r"samples: \[{}, {}, {}\]".format(
                          _num_samples(y1), _num_samples(y2),
                          _num_samples(sample_weight) * 2))
-    assert_raise_message(ValueError, error_message, metric, y1, y2,
-                         sample_weight=np.hstack([sample_weight,
-                                                  sample_weight]))
+    with pytest.raises(ValueError, match=error_message):
+        metric(y1, y2, sample_weight=np.hstack([sample_weight,
+                                                sample_weight]))
 
 
 @pytest.mark.parametrize(
-        'name',
-        (set(ALL_METRICS).intersection(set(REGRESSION_METRICS))
-         - METRICS_WITHOUT_SAMPLE_WEIGHT))
+    'name',
+    sorted(
+        set(ALL_METRICS).intersection(set(REGRESSION_METRICS)) -
+        METRICS_WITHOUT_SAMPLE_WEIGHT))
 def test_regression_sample_weight_invariance(name):
     n_samples = 50
     random_state = check_random_state(0)
@@ -1150,9 +1307,10 @@ def test_regression_sample_weight_invariance(name):
 
 
 @pytest.mark.parametrize(
-        'name',
-        (set(ALL_METRICS) - set(REGRESSION_METRICS)
-         - METRICS_WITHOUT_SAMPLE_WEIGHT - METRIC_UNDEFINED_BINARY))
+    'name',
+    sorted(
+        set(ALL_METRICS) - set(REGRESSION_METRICS) -
+        METRICS_WITHOUT_SAMPLE_WEIGHT - METRIC_UNDEFINED_BINARY))
 def test_binary_sample_weight_invariance(name):
     # binary
     n_samples = 50
@@ -1168,10 +1326,10 @@ def test_binary_sample_weight_invariance(name):
 
 
 @pytest.mark.parametrize(
-        'name',
-        (set(ALL_METRICS) - set(REGRESSION_METRICS)
-         - METRICS_WITHOUT_SAMPLE_WEIGHT
-         - METRIC_UNDEFINED_BINARY_MULTICLASS))
+    'name',
+    sorted(
+        set(ALL_METRICS) - set(REGRESSION_METRICS) -
+        METRICS_WITHOUT_SAMPLE_WEIGHT - METRIC_UNDEFINED_BINARY_MULTICLASS))
 def test_multiclass_sample_weight_invariance(name):
     # multiclass
     n_samples = 50
@@ -1181,23 +1339,26 @@ def test_multiclass_sample_weight_invariance(name):
     y_score = random_state.random_sample(size=(n_samples, 5))
     metric = ALL_METRICS[name]
     if name in THRESHOLDED_METRICS:
-        check_sample_weight_invariance(name, metric, y_true, y_score)
+        # softmax
+        temp = np.exp(-y_score)
+        y_score_norm = temp / temp.sum(axis=-1).reshape(-1, 1)
+        check_sample_weight_invariance(name, metric, y_true, y_score_norm)
     else:
         check_sample_weight_invariance(name, metric, y_true, y_pred)
 
 
 @pytest.mark.parametrize(
-        'name',
-        (MULTILABELS_METRICS | THRESHOLDED_MULTILABEL_METRICS |
-         MULTIOUTPUT_METRICS) - METRICS_WITHOUT_SAMPLE_WEIGHT)
+    'name',
+    sorted((MULTILABELS_METRICS | THRESHOLDED_MULTILABEL_METRICS
+            | MULTIOUTPUT_METRICS) - METRICS_WITHOUT_SAMPLE_WEIGHT))
 def test_multilabel_sample_weight_invariance(name):
     # multilabel indicator
     random_state = check_random_state(0)
-    _, ya = make_multilabel_classification(n_features=1, n_classes=20,
-                                           random_state=0, n_samples=100,
+    _, ya = make_multilabel_classification(n_features=1, n_classes=10,
+                                           random_state=0, n_samples=50,
                                            allow_unlabeled=False)
-    _, yb = make_multilabel_classification(n_features=1, n_classes=20,
-                                           random_state=1, n_samples=100,
+    _, yb = make_multilabel_classification(n_features=1, n_classes=10,
+                                           random_state=1, n_samples=50,
                                            allow_unlabeled=False)
     y_true = np.vstack([ya, yb])
     y_pred = np.vstack([ya, ya])
@@ -1235,7 +1396,8 @@ def test_no_averaging_labels():
 
 
 @pytest.mark.parametrize(
-    'name', MULTILABELS_METRICS - {"unnormalized_multilabel_confusion_matrix"})
+    'name',
+    sorted(MULTILABELS_METRICS - {"unnormalized_multilabel_confusion_matrix"}))
 def test_multilabel_label_permutations_invariance(name):
     random_state = check_random_state(0)
     n_samples, n_classes = 20, 4
@@ -1255,7 +1417,7 @@ def test_multilabel_label_permutations_invariance(name):
 
 
 @pytest.mark.parametrize(
-    'name', THRESHOLDED_MULTILABEL_METRICS | MULTIOUTPUT_METRICS)
+    'name', sorted(THRESHOLDED_MULTILABEL_METRICS | MULTIOUTPUT_METRICS))
 def test_thresholded_multilabel_multioutput_permutations_invariance(name):
     random_state = check_random_state(0)
     n_samples, n_classes = 20, 4
@@ -1274,5 +1436,96 @@ def test_thresholded_multilabel_multioutput_permutations_invariance(name):
         y_score_perm = y_score[:, perm]
         y_true_perm = y_true[:, perm]
 
+        current_score = metric(y_true_perm, y_score_perm)
+        if metric == mean_absolute_percentage_error:
+            assert np.isfinite(current_score)
+            assert current_score > 1e6
+            # Here we are not comparing the values in case of MAPE because
+            # whenever y_true value is exactly zero, the MAPE value doesn't
+            # signify anything. Thus, in this case we are just expecting
+            # very large finite value.
+        else:
+            assert_almost_equal(score, current_score)
+
+
+@pytest.mark.parametrize(
+    'name',
+    sorted(set(THRESHOLDED_METRICS) - METRIC_UNDEFINED_BINARY_MULTICLASS))
+def test_thresholded_metric_permutation_invariance(name):
+    n_samples, n_classes = 100, 3
+    random_state = check_random_state(0)
+
+    y_score = random_state.rand(n_samples, n_classes)
+    temp = np.exp(-y_score)
+    y_score = temp / temp.sum(axis=-1).reshape(-1, 1)
+    y_true = random_state.randint(0, n_classes, size=n_samples)
+
+    metric = ALL_METRICS[name]
+    score = metric(y_true, y_score)
+    for perm in permutations(range(n_classes), n_classes):
+        inverse_perm = np.zeros(n_classes, dtype=int)
+        inverse_perm[list(perm)] = np.arange(n_classes)
+        y_score_perm = y_score[:, inverse_perm]
+        y_true_perm = np.take(perm, y_true)
+
         current_score = metric(y_true_perm, y_score_perm)
         assert_almost_equal(score, current_score)
+
+
+@pytest.mark.parametrize("metric_name", CLASSIFICATION_METRICS)
+def test_metrics_consistent_type_error(metric_name):
+    # check that an understable message is raised when the type between y_true
+    # and y_pred mismatch
+    rng = np.random.RandomState(42)
+    y1 = np.array(["spam"] * 3 + ["eggs"] * 2, dtype=object)
+    y2 = rng.randint(0, 2, size=y1.size)
+
+    err_msg = "Labels in y_true and y_pred should be of the same type."
+    with pytest.raises(TypeError, match=err_msg):
+        CLASSIFICATION_METRICS[metric_name](y1, y2)
+
+
+@pytest.mark.parametrize(
+    "metric, y_pred_threshold",
+    [
+        (average_precision_score, True),
+        (brier_score_loss, True),
+        (f1_score, False),
+        (partial(fbeta_score, beta=1), False),
+        (jaccard_score, False),
+        (precision_recall_curve, True),
+        (precision_score, False),
+        (recall_score, False),
+        (roc_curve, True),
+    ],
+)
+@pytest.mark.parametrize("dtype_y_str", [str, object])
+def test_metrics_pos_label_error_str(metric, y_pred_threshold, dtype_y_str):
+    # check that the error message if `pos_label` is not specified and the
+    # targets is made of strings.
+    rng = np.random.RandomState(42)
+    y1 = np.array(["spam"] * 3 + ["eggs"] * 2, dtype=dtype_y_str)
+    y2 = rng.randint(0, 2, size=y1.size)
+
+    if not y_pred_threshold:
+        y2 = np.array(["spam", "eggs"], dtype=dtype_y_str)[y2]
+
+    err_msg_pos_label_None = (
+        "y_true takes value in {'eggs', 'spam'} and pos_label is not "
+        "specified: either make y_true take value in {0, 1} or {-1, 1} or "
+        "pass pos_label explicit"
+    )
+    err_msg_pos_label_1 = (
+        r"pos_label=1 is not a valid label. It should be one of "
+        r"\['eggs', 'spam'\]"
+    )
+
+    pos_label_default = signature(metric).parameters["pos_label"].default
+
+    err_msg = (
+        err_msg_pos_label_1
+        if pos_label_default == 1
+        else err_msg_pos_label_None
+    )
+    with pytest.raises(ValueError, match=err_msg):
+        metric(y1, y2)
